@@ -86,6 +86,7 @@ typedef enum { /*< skip >*/
 	GTH_BROWSER_HEADER_SECTION_BROWSER_LOCATIONS,
 	GTH_BROWSER_HEADER_SECTION_BROWSER_COMMANDS,
 	GTH_BROWSER_HEADER_SECTION_BROWSER_TOOLS,
+	GTH_BROWSER_HEADER_SECTION_BROWSER_METADATA_TOOLS,
 	GTH_BROWSER_HEADER_SECTION_BROWSER_VIEW,
 	GTH_BROWSER_HEADER_SECTION_VIEWER_NAVIGATION,
 	GTH_BROWSER_HEADER_SECTION_VIEWER_OTHER_VIEW,
@@ -95,6 +96,7 @@ typedef enum { /*< skip >*/
 	GTH_BROWSER_HEADER_SECTION_VIEWER_OTHER_COMMANDS,
 	GTH_BROWSER_HEADER_SECTION_VIEWER_SIDEBAR,
 	GTH_BROWSER_HEADER_SECTION_VIEWER_EDIT,
+	GTH_BROWSER_HEADER_SECTION_VIEWER_EDIT_METADATA,
 	GTH_BROWSER_HEADER_SECTION_EDITOR_NAVIGATION,
 	GTH_BROWSER_HEADER_SECTION_EDITOR_VIEW,
 	GTH_BROWSER_HEADER_SECTION_EDITOR_COMMANDS,
@@ -107,10 +109,16 @@ typedef enum {
 	GTH_ACTION_GO_BACK,
 	GTH_ACTION_GO_FORWARD,
 	GTH_ACTION_GO_UP,
-	GTH_ACTION_LIST_CHILDREN,
-	GTH_ACTION_VIEW
+	GTH_ACTION_TREE_LIST_CHILDREN,
+	GTH_ACTION_TREE_OPEN
 } GthAction;
 
+typedef enum {
+	GTH_SCROLL_ACTION_NONE,
+	GTH_SCROLL_ACTION_CHANGE_FILE,
+	GTH_SCROLL_ACTION_ZOOM
+} GthScrollAction;
+
 struct _GthBrowser
 {
 	GthWindow __parent;
@@ -248,6 +256,8 @@ void             gth_browser_update_statusbar_file_info
 						    (GthBrowser       *browser);
 void             gth_browser_show_file_properties   (GthBrowser       *browser);
 void             gth_browser_show_viewer_tools      (GthBrowser       *browser);
+void             gth_browser_toggle_file_properties (GthBrowser       *browser);
+void             gth_browser_toggle_viewer_tools    (GthBrowser       *browser);
 void             gth_browser_hide_sidebar           (GthBrowser       *browser);
 void             gth_browser_load_location          (GthBrowser       *browser,
 						     GFile            *location);
@@ -278,12 +288,10 @@ GtkWidget *      gth_browser_get_fullscreen_headerbar
 						    (GthBrowser	      *browser);
 void             gth_browser_keep_mouse_visible     (GthBrowser	      *browser,
 						     gboolean          value);
+void             gth_browser_show_menu              (GthBrowser	      *browser);
 
 /* utilities */
 
-GMenuItem *	_g_menu_item_new_for_file	    (GFile            *file,
-						     const char       *custom_label);
-
 G_END_DECLS
 
 #endif /* GTH_BROWSER_H */
