@@ -24,6 +24,7 @@
 
 #include <gtk/gtk.h>
 #include "gtk-utils.h"
+#include "gth-shortcut.h"
 
 G_BEGIN_DECLS
 
@@ -88,6 +89,10 @@ void           gth_window_show_only_content  (GthWindow     *window,
 					      gboolean       only_content);
 GtkWidget *    gth_window_get_area           (GthWindow     *window,
 					      GthWindowArea  area);
+void           gth_window_add_overlay        (GthWindow     *window,
+					      GtkWidget     *widget);
+void           gth_window_set_header_bar     (GthWindow     *window,
+					      GtkWidget     *header_bar);
 GtkWidget *    gth_window_get_header_bar     (GthWindow     *window);
 void           gth_window_save_page_size     (GthWindow     *window,
 					      int            page,
@@ -103,7 +108,7 @@ void           gth_window_clear_saved_size   (GthWindow     *window,
       	      	      	      	      	      int            page);
 void           gth_window_set_title          (GthWindow     *window,
 					      const char    *title,
-					      const char    *subtitle);
+					      GList	    *emblems);
 GtkAccelGroup *gth_window_get_accel_group	(GthWindow		*window);
 void           gth_window_add_accelerators	(GthWindow		*window,
 						 const GthAccelerator	*accelerators,
@@ -116,6 +121,40 @@ gboolean	gth_window_get_action_state	(GthWindow		*window,
 void		gth_window_change_action_state	(GthWindow		*window,
 						 const char		*action_name,
 						 gboolean		 value);
+void		gth_window_add_viewer_shortcuts (GthWindow              *window,
+						 const char             *viewer_context,
+						 const GthShortcut      *shortcuts,
+						 int                     n_shortcuts);
+void		gth_window_add_shortcuts	(GthWindow		*window,
+						 const GthShortcut	*shortcuts,
+						 int			 n_shortcuts);
+GPtrArray *	gth_window_get_shortcuts	(GthWindow		*window);
+GthShortcut *   gth_window_get_shortcut	(GthWindow		*window,
+						 const char             *detailed_action);
+GPtrArray *	gth_window_get_shortcuts_by_category
+						(GthWindow		*window);
+gboolean	gth_window_activate_shortcut	(GthWindow		*window,
+						 int			 context,
+						 const char             *viewer_context,
+						 guint			 keycode,
+						 GdkModifierType	 modifiers);
+void		gth_window_load_shortcuts	(GthWindow		*window);
+void		gth_window_add_removable_shortcut
+						(GthWindow		*window,
+						 const char		*group_name,
+						 GthShortcut		*shortcut);
+void		gth_window_remove_shortcuts	(GthWindow		*window,
+						 const char		*group_name);
+gboolean	gth_window_can_change_shortcut	(GthWindow		*window,
+						 const char		*detailed_action,
+						 int			 context,
+						 const char		*viewer_context,
+						 guint			 keycode,
+						 GdkModifierType	 modifiers,
+						 GtkWindow		*parent);
+void		gth_window_copy_shortcuts	(GthWindow		*to_window,
+						 GthWindow		*from_window,
+						 int			 context);
 
 G_END_DECLS
 
